/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.readers.its.trac;

// Bridge API from com.rempl.readers.its:bridge-its-api
import com.rempl.readers.its.bridge.Change;

import com.ymock.util.Logger;

/**
 * Builder of {@link Change} from {@link TracTuple}. It is
 * a Chain of Responsibility pattern.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: AbstractChangeBuilder.java 1614 2011-07-10 19:05:43Z guard $
 */
abstract class AbstractChangeBuilder implements ChangeBuilder {

    /**
     * Next builder, if set.
     */
    private ChangeBuilder next;

    /**
     * Create and return a chain of builders.
     * @return The chain
     */
    public static ChangeBuilder chain() {
        return new CbTitle()
            .add(new CbMessage())
            .add(new CbStatus())
            .add(new CbAssignee())
            .add(new CbLegallyIgnore());
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public final ChangeBuilder add(final ChangeBuilder builder) {
        if (this.next == null) {
            this.next = builder;
        } else {
            this.next.add(builder);
        }
        return this;
    }

    /**
     * Descends down the chain and looks for the appropriate builder.
     * Each builder along the path has a chance to build the Change object.
     * If it succeeds, the object is returned. If it doesn't (by returning
     * null), next builder (that is referenced in <b>next</b> field)
     * is given a chance.
     *
     * <p>E.g. here we have four builders, the first two are not the right ones
     * for the given TracTuple, the third one gives a good response (a Change3
     * object), so the last fourth one is not even tried - the result of the
     * build method called on the Builder1 is the Change3 object.</p>
     * <pre>
     * (TracTuple) ==&gt; Builder1 ===&gt; null
     *                               ||
     *                          (TracTuple)
     *                               ||
     *                            Builder2 ===&gt; null
     *                                       ||
     *                                   (TracTuple)
     *                                       ||
     *                                    Builder3 ===&gt; <b>Change3</b>
     *                                       |
     *                                       |
     *                                    Builder4 ---&gt; ?
     * </pre>
     *
     * @param tuple The tuple to work with
     * @return The change just built or NULL if nothing done
     */
    @Override
    public final Change build(final TracTuple tuple) {
        Change change = this.buildIt(tuple);
        if (change == null && this.next != null) {
            change = this.next.build(tuple);
        }
        if (change != null) {
            Logger.debug(
                this,
                "#build(%s): %s discovered",
                tuple,
                change.getClass().getName()
            );
        }
        return change;
    }

    /**
     * Abstract method to build new {@link Change} from
     * the provided {@link TracTuple}.
     * @param tuple The tuple to work with
     * @return The change just built or NULL if nothing done
     */
    protected abstract Change buildIt(final TracTuple tuple);

}
