/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.readers.its.trac;

// Bridge API from com.rempl.readers.its:bridge-its-api
import com.rempl.readers.its.bridge.BridgeException;
import com.rempl.readers.its.bridge.Change;
import com.rempl.readers.its.bridge.Issue;
import com.rempl.readers.its.bridge.Person;

// bridge utils from com.rempl.readers:readers-bridge-utils
import com.rempl.readers.utils.RpcAdapter;

import com.ymock.util.Logger;

// for manipulation of changes
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Map;

/**
 * One issue.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: TracIssue.java 1662 2011-07-18 19:45:01Z guard $
 * @see <a href="http://trac-hacks.org/wiki/XmlRpcPlugin">XmlRpcPlugin</a>
 * @todo #19 We need to implement some thread-wide caching
 *       of retrieved issues.
 */
final class TracIssue implements Issue {

    /**
     * Position of date-of-change in the log tuple.
     */
    private static final Integer LOG_DATE = 0;

    /**
     * Position of author-of-change in the log tuple.
     */
    private static final Integer LOG_AUTHOR = 1;

    /**
     * Position of key-of-change in the log tuple.
     */
    private static final Integer LOG_KEY = 2;

    /**
     * Position of value-of-change in the log tuple.
     */
    private static final Integer LOG_VALUE = 4;

    /**
     * Position of date-of-creation in the ticket details.
     */
    private static final Integer ATTR_DATE = 1;

    /**
     * Position of value-map in the log tuple.
     */
    private static final Integer ATTR_MAP = 3;

    /**
     * Remote Procedure Call adapter.
     */
    private final RpcAdapter rpc;

    /**
     * Ticket ID.
     */
    private final Integer ticket;

    /**
     * Collection of changes.
     */
    private Collection<Change> changes;

    /**
     * Public ctor.
     * @param adapter The RPC adapter
     * @param tkt Ticket number
     * @see XmlRpcBridge#retrieve()
     * @throws BridgeException If some failure happens
     */
    public TracIssue(final RpcAdapter adapter, final Integer tkt)
        throws BridgeException {
        this.rpc = adapter;
        this.ticket = tkt;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Collection<Change> changes() throws BridgeException {
        if (this.changes == null) {
            this.changes = new ArrayList<Change>();
            final ChangeBuilder builder = AbstractChangeBuilder.chain();
            this.changes.addAll(this.fetchAttribs(builder));
            this.changes.addAll(this.fetchLog(builder));
            Logger.info(
                this,
                "#changes(#%d): %d changes loaded",
                this.ticket,
                this.changes.size()
            );
        }
        return this.changes;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public <T extends Change> T latest(final Class<T> type)
        throws BridgeException {
        Change latest = null;
        for (Change change : this.changes()) {
            final boolean matches = change.getClass().equals(type)
                && (latest == null || latest.time().before(change.time()));
            if (matches) {
                latest = change;
            }
        }
        assert latest != null;
        return (T) latest;
    }

    /**
     * Fetch information from the ticket attributes.
     * @param builder ChangeBuilder to use for building {@link Change}s
     * @return Collection of changes
     * @throws BridgeException If some failure happens
     * @see #changes()
     */
    private Collection<Change> fetchAttribs(final ChangeBuilder builder)
        throws BridgeException {
        final Collection<Change> chgs = new ArrayList<Change>();
        final Object[] summary =
            (Object[]) this.rpc.call("ticket.get", this.ticket);
        final Date created = (Date) summary[this.ATTR_DATE];
        final Map<Object, Object> attribs =
            (Map<Object, Object>) summary[this.ATTR_MAP];
        final Person author = new Person((String) attribs.get("reporter"));
        for (Map.Entry<Object, Object> pair : attribs.entrySet()) {
            final Change change = builder.build(
                new TracTuple(
                    created,
                    author,
                    (String) pair.getKey(),
                    pair.getValue()
                )
            );
            if (change != null) {
                chgs.add(change);
            }
        }
        Logger.debug(
            this,
            "#fetchAttribs() at #%d: %d changes loaded",
            this.ticket,
            chgs.size()
        );
        return chgs;
    }

    /**
     * Fetch information from the ticket log.
     * @param builder ChangeBuilder to use for building {@link Change}s
     * @return Collection of changes
     * @throws BridgeException If some failure happens
     * @see #changes()
     */
    private Collection<Change> fetchLog(final ChangeBuilder builder)
        throws BridgeException {
        final Collection<Change> chgs = new ArrayList<Change>();
        final Object[] tuples =
            (Object[]) this.rpc.call("ticket.changeLog", this.ticket);
        for (Object object : tuples) {
            final Object[] tuple = (Object[]) object;
            final Change change = builder.build(
                new TracTuple(
                    (Date) tuple[this.LOG_DATE],
                    new Person((String) tuple[this.LOG_AUTHOR]),
                    (String) tuple[this.LOG_KEY],
                    (String) tuple[this.LOG_VALUE]
                )
            );
            if (change != null) {
                chgs.add(change);
            }
        }
        Logger.debug(
            this,
            "#fetchLog() at #%d: %d changes loaded",
            this.ticket,
            chgs.size()
        );
        return chgs;
    }

}
