/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.readers.its.trac;

// Bridge API from com.rempl.readers.its:bridge-its-api
import com.rempl.readers.its.bridge.Bridge;
import com.rempl.readers.its.bridge.BridgeException;
import com.rempl.readers.its.bridge.Issue;

// bridge utils from com.rempl.readers:readers-bridge-utils
import com.rempl.readers.utils.RpcAdapter;

import com.ymock.util.Logger;

// for manipulation of issues
import java.util.HashMap;
import java.util.Map;

/**
 * Bridge between Trac and REMPL Reader.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: XmlRpcBridge.java 1626 2011-07-11 19:57:33Z guard $
 * @see <a href="http://trac-hacks.org/wiki/XmlRpcPlugin">XmlRpcPlugin</a>
 */
public final class XmlRpcBridge implements Bridge {

    /**
     * Parameter name to use for setting URL of Trac.
     */
    public static final String KEY_URL = "url";

    /**
     * Parameter name to use for setting factory.
     */
    private static final String KEY_FACTORY = "transportFactory";

    /** Parameter name to use for setting user name. */
    private static final String KEY_USER = "user";

    /** Parameter name to use for setting user password. */
    private static final String KEY_PASSWORD = "password";

    /**
     * Remote Procedure Call adapter.
     */
    private RpcAdapter rpc;

    /**
     * Public ctor.
     */
    public XmlRpcBridge() {
        this.rpc = null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void configure(final Map<String, String> params)
        throws BridgeException {
        if (!params.containsKey(this.KEY_URL)) {
            throw new BridgeException(
                "Configuration parameter \""
                + this.KEY_URL
                + "\" is mandatory"
            );
        }
        this.rpc = new RpcAdapter(params.get(this.KEY_URL), "/rpc");
        if (params.containsKey(this.KEY_FACTORY)) {
            this.rpc.setTransportFactory(params.get(this.KEY_FACTORY));
        }
        if (params.containsKey(this.KEY_USER)
            && params.containsKey(this.KEY_PASSWORD)) {
            this.rpc.setAuth(params.get(this.KEY_USER),
                params.get(this.KEY_PASSWORD));
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Map<String, Issue> retrieve() throws BridgeException {
        if (this.rpc == null) {
            throw new IllegalStateException("Use configure() first");
        }
        final Map<String, Issue> issues = new HashMap<String, Issue>();
        final Object[] tickets =
            (Object[]) this.rpc.call("ticket.query", "max=0");
        for (Object ticket : tickets) {
            issues.put(
                ticket.toString(),
                new TracIssue(this.rpc, (Integer) ticket)
            );
        }
        Logger.info(
            this,
            "#retrieve(): %d tickets loaded from Trac",
            issues.size()
        );
        return issues;
    }

}
